"""
The exact sub-package




"""

from ctypes import cdll
from pkg_resources import resource_filename
import sys

try:
    _libexact = cdll.LoadLibrary(resource_filename('pysplashsph', 'libs/libexact.so'))
except OSError:
    print("PySsplashSPH ERROR: Could not load `libexact.so`")
    sys.exit(1)

from .exact import (shock, shock_sr, sedov, polytrope,
                    toystar1D, toystar2D, gresho, rhoh,
                    torus, ringspread, dustywave, rochelobe,
                    cshock, planetdisc, bondi)

__all__ = ['shock', 'shock_sr', 'sedov', 'polytrope',
           'toystar1D', 'toystar2D', 'gresho', 'rhoh',
           'torus', 'ringspread', 'dustywave', 'rochelobe',
           'cshock', 'planetdisc', 'bondi']



import pysplashsph
import numpy as np
import os

test_dir = os.path.dirname(os.path.realpath(__file__))
test_file_ascii  = os.path.join(test_dir, 'data/test_00000.ascii')
test_file_binary = os.path.join(test_dir, 'data/test_00000')

def test_read(capfd):

    print("Loading:",test_file_ascii)
    sph_data_from_ascii = np.genfromtxt(test_file_ascii).T

    print("Loading:",test_file_binary)
    dump = pysplashsph.read.read_data(test_file_binary, filetype='Phantom')
    sph_data = dump.data

    capfd.readouterr()  # capture OS level output, so that it can be silenced with "pytest -s"

    # Don't check if they are exactly equal, since ascii data is rounded
    if np.allclose(sph_data_from_ascii, sph_data):
        success = True
    else:
        success = False
        print("\nERROR in loading data! Values don't match.\n")
        print("Indices where they don't match:")
        w = np.invert(np.isclose(sph_data_from_ascii, sph_data))
        print(np.argwhere(w))
        print("\nBad values:")
        print(sph_data[w])
        print("\nCorrect values:")
        print(sph_data_from_ascii[w])

    assert success, 'Binary data does not match ascii data.'



import pysplash
import os
import h5py
import matplotlib.pyplot as plt

test_dir = os.path.dirname(os.path.realpath(__file__))

test_file_binary = os.path.join(test_dir, 'data/fulldump_00000')

data_binary = pysplash.read.read_data(test_file_binary, filetype='Phantom')

data_binary_as_hdf5 = data_binary.as_hdf5

test_snap = "test_save.h5"

data_binary.save_hdf5(test_snap)


import plonk
snap = plonk.load_snap(test_snap)

snap.set_units(position='au', density='g/cm^3', projection='cm')

snap.image(
    quantity='density',
    x='x',
    y='z',
    interp='slice',
    cmap='gist_heat',
)

plt.show()



"""
The read sub-package

This sub-package will read many different SPH data formats, and return
their contents as a numpy array

"""

from ctypes import cdll
from pkg_resources import resource_filename
import sys

try:
    _libread = cdll.LoadLibrary(resource_filename('pysplashsph', 'libs/libread.so'))
except OSError:
    print("PySplashSPH ERROR: Could not load `libread.so`")
    sys.exit(1)

from .read import read_data

__all__ = ['read_data']



import pysplashsph
import numpy as np

def test_shock(capfd):
  x = np.linspace(0, 1, 10)
  y = pysplashsph.exact.shock(x)
  capfd.readouterr()  # capture OS level output, so that it can be silenced with "pytest -s"



import os.path
import tempfile

# from pathlib import Path

from ctypes import (c_int, c_float, c_bool, c_double, c_char_p,
                        byref, POINTER, pointer, cast, c_char)

from . import _libread as libread
from ..utils import stdchannel_redirected
import numpy as np
import h5py
from pandas import DataFrame
import time

# Global constants that specify the length of strings in some of the
# SPLASH subroutines. Changing these could break the code and lead to a
# Segmentation Fault
ltags = 16
lenlabel = 80
lenunitslabel = 40


class Dump:
    """PySPLASH Dump object. Contains SPH data, labels, units, and other
    attributes.


    """

    def __init__(self, data=None, labels=None, headers=None, filepath=None, filetype=None):
        self.data = data
        self.labels = labels
        self.headers = headers
        self.filepath = filepath
        self.filetype = filetype

        self._as_dataframe = None
        self._as_hdf5 = None

    def __getitem__(self, name):
        if name in ['headers', 'header']:
            return self.headers

        if name in self.labels:
            try:
                col = self.labels.index(name)
            except ValueError:
                print("Label {} not a column name in dumpfile".format(name))
                raise

            return self.data[col]

        if self.as_dataframe is not None:
            return self.as_dataframe[name].values
        else:
            self._to_dataframe()
            return self.as_dataframe[name].values

    @property
    def as_dataframe(self):
        if self._as_dataframe is None:
            self._as_dataframe = DataFrame(data=self.data, columns=self.labels)
        return self._as_dataframe

    @property
    def as_hdf5(self):
        if self._as_hdf5 is None:
            self._as_hdf5 = self._to_hdf5_dataset()
        return self._as_hdf5

    def _to_hdf5_dataset(self):
        fname = os.path.basename(self.filepath) + b'.h5'
        tf = tempfile.NamedTemporaryFile()
        f = h5py.File(tf, 'w')
        f = _dump2hdf5(f, self)
        return f

    def save_hdf5(self, filename):
        print("Warning: save_hdf5 should be avoided. If you are trying to save ")
        print("a Phantom file as HDF5, use the phantom2hdf5 utility in Phantom.")

        dump_hdf5 = self.as_hdf5

        with h5py.File(filename, "w") as f:
            for group_name in dump_hdf5:
                f_a = f.create_group(group_name)
                for dataset in dump_hdf5[group_name]:
                    f_a.create_dataset(dataset, data=dump_hdf5[group_name][dataset])


def _dump2hdf5(f, dump):
    """ Convert the attributes of the Dump class into an HDF5 File object. """

    if dump.headers is not None:
        f_header = f.create_group('header')
        for header in dump.headers:
            f_header.create_dataset(header, data=dump.headers[header])

    if dump.filetype.lower().decode() == 'phantom':
        return _phantom2hdf5(f, dump)

    f_particles = f.create_group('particles')

    _store_remainder(f_particles, dump)

    return f


def _phantom2hdf5(f, dump):
    """ A simple tool for converting to HDF5 format for use in other codes (e.g. Plonk)

    NOTE: This is not a replacement for phantom2hdf5, and not all of the contents
          are stored in these files. They do not contain enough information to
          restart a Phantom simulation.
    """

    _store_extra_header_quantities(f['header'], dump)

    # Mask out the sink particles so we can store them in a different Group
    sink_mask = dump['itype'] == 3
    sink_indices = np.where(sink_mask)

    ignore_labels = []

    if len(sink_indices) > 0:
        f_sinks = f_particles = f.create_group('sinks')

        # Ignore x, y, z, since they are added into the xyz group
        ignore_labels.extend(['x', 'y', 'z'])

        sink_xyz_data = np.array([dump['x'][sink_indices], dump['y'][sink_indices], dump['z'][sink_indices]]).T
        f_sinks.create_dataset('xyz', data=sink_xyz_data)

        if 'vx' in dump.labels:
            # If velocity is present, it is a full dump
            _store_fulldump(f_sinks, dump, mask=sink_mask, ignore_labels=ignore_labels)

        _store_remainder(f_sinks, dump, mask=sink_mask, ignore_labels=ignore_labels)


    # Find particles based off our sink mask made earlier
    particle_mask = np.logical_not(sink_mask)
    particle_indices = np.where(particle_mask)

    f_particles = f.create_group('particles')

    # Ignore x, y, z, since they are added into the xyz group
    ignore_labels.extend(['x', 'y', 'z'])

    particle_xyz_data = np.array([dump['x'][particle_indices], dump['y'][particle_indices], dump['z'][particle_indices]]).T
    f_particles.create_dataset('xyz', data=particle_xyz_data)

    if 'vx' in dump.labels:
        _store_fulldump(f_particles, dump, mask=particle_mask, ignore_labels=ignore_labels)

    _store_remainder(f_particles, dump, mask=particle_mask, ignore_labels=ignore_labels)

    return f


def _store_fulldump(f_a, dump, mask=None, ignore_labels=[]):
    """ Store velocity components of a fulldump """
    f_a.create_dataset('vxyz', data=np.array([dump['vx'][mask], dump['vy'][mask], dump['vz'][mask]]))
    f_a.create_dataset('divv', data=dump['divv'][mask])
    f_a.create_dataset('dt', data=dump['dt'][mask])

    if len(ignore_labels) > 0:
        # Ignore these quantites since they are added into groups
        ignore_labels.extend(['vx', 'vy', 'vz', 'divv', 'dt'])


def _store_remainder(f_a, dump, mask=None, ignore_labels=[]):
    """ Store additional labels from dump if they are not already in the HDF5 file object """
    for label in dump.labels:
        if label not in (f_a.keys() and ignore_labels):
            f_a.create_dataset(label, data=np.array(dump[label][mask]))


def _store_extra_header_quantities(f_a, dump):
    """ Store some extra quantities that Plonk needs. """
    if 'ieos' not in f_a.keys():
        f_a.create_dataset('ieos', data=3)

    if 'massoftype' not in f_a.keys():
        massoftype = []
        for key in f_a.keys():
            if 'massoftype' in key:
                massoftype.append(f_a[key][()])

        f_a.create_dataset('massoftype', data=np.array(massoftype))


def read_data(filepath, filetype='Phantom', use_HDF5=False,
                     ncol=None, npart=None, verbose=False):
    """Generate a Snap object from a Phantom HDF5 file.
    Parameters
    ----------
    filepath
        The path to the file.
    filetype
        The format of the file. Will load HDF5 files if specified,
        otherwise a binary data format supported by SPLASH can be read
    use_HDF5
        Boolean to specify whether to use the HDF5 library to read files.
    ncol, npart
        The number of columns and particles in the data. Incorrectly
        specifying these can lead to a Segmentation Fault
    verbose
        Specify if you want libread to provide output to the terminal

    Returns
    -------
    Dump
        A Dump object.
    """

    if not os.path.exists(filepath):
        raise FileNotFoundError("filepath " + str(filepath) + " does not exist.")

    extension = os.path.splitext(filepath)

    if use_HDF5 is True:
        #any(ext in ['h5', 'hdf5'] for ext in [filetype.lower(), extension]) or \
        #                    use_HDF5 is True:
        return read_hdf5(filepath)

    else:
        return read_data_binary(filepath, filetype=filetype,
                             ncol=None, npart=None, verbose=False)


def read_hdf5(filepath):
    if not h5py.is_hdf5(filepath):
        raise TypeError("File given is not an HDF5 file.")

    return h5py.File(filepath, mode='r')


def read_data_binary(filepath, filetype='Phantom',
                     ncol=None, npart=None, verbose=False):

    filepath = filepath.encode('utf-8')
    filetype = filetype.encode('utf-8')

    f_length = c_int(len(filepath))
    ff_length = c_int(len(filetype))

    # set up verbosity
    if verbose:
        verbose_int = c_int(1)
        print_warnings = True
    else:
        print_warnings = False
        verbose_int = c_int(0)

    # We need to know the amount of memory to allocate. If this is not yet
    # given, we need to find out how much to allocate.
    if ncol is None or npart is None:
        # ncol and npart will be obtained from our first call to libread.read_data
        # This first call will not result in the data actually being loaded
        # for some data formats, e.g.
        # Phantom, SPHng, gadget, and others,

        ncol_in = 0
        npart_in = 0

        ncol_in_c = c_int(ncol_in)
        npart_in_c = c_int(npart_in)

        # Fortran read_data subroutine arguments are:
        # filename,fileformat,f_length, ff_length,&
        # sph_dat,npart,ncol,read_header,verbose,ierr

        # Tell ctypes what data types we are going to send to our Fortran code
        libread.read_data.argtypes = \
                [c_char_p, c_char_p, POINTER(c_int), POINTER(c_int),
                 POINTER(c_double * npart_in * ncol_in), POINTER(c_int), POINTER(c_int),
                 POINTER(c_int), POINTER(c_int), POINTER(c_int)]

        sph_dat = (c_double * npart_in * ncol_in)() # Order of npart and ncol matters here

        # Capture error flag with an integer
        ierr = c_int(0)

        # Indicate that we just want to read the header to get ncol and npart
        read_header = c_int(1)

        with stdchannel_redirected():
            libread.read_data(c_char_p(filepath), c_char_p(filetype),
                              byref(f_length), byref(ff_length),
                              byref(sph_dat),
                              byref(npart_in_c), byref(ncol_in_c),
                              byref(read_header), byref(verbose_int), byref(ierr))


        if verbose: print("Got file size, ncol="+ str(ncol_in_c) +
                            ", npart=" + str(npart_in_c))

        ncol = ncol_in_c
        npart = npart_in_c

    else:
        # If ncol and npart are given, convert to c_int
        npart = c_int(npart)
        ncol = c_int(ncol)

    # Update argtypes based on array size
    libread.read_data.argtypes = \
            [c_char_p, c_char_p, POINTER(c_int), POINTER(c_int),
             POINTER(c_double * npart.value * ncol.value), POINTER(c_int), POINTER(c_int),
             POINTER(c_int), POINTER(c_int), POINTER(c_int)]

    sph_dat = (c_double * npart.value * ncol.value)()

    ierr = c_int(0)
    read_header = c_int(0)

    with stdchannel_redirected(print_warnings=print_warnings):
        libread.read_data(c_char_p(filepath), c_char_p(filetype), # strings
                              byref(f_length), byref(ff_length), # length of previous strings
                              byref(sph_dat), # An array with the size of the SPH data
                              byref(npart), byref(ncol), # the size of sph_dat
                              byref(read_header), byref(verbose_int), byref(ierr))

    if ierr == 1:
        raise RuntimeError("Error encountered in SPLASH.")

    # Turn data into a numpy array
    sph_data = np.ctypeslib.as_array(sph_dat) # .T

    labels = get_labels(ncol.value-1) # subtract 1 since itype is not included

    labels.append('itype') # Last column is always particle type
    header_tags, header_vals = get_headers()

    dump = Dump()
    dump.filepath = filepath
    dump.filetype = filetype
    dump.data = sph_data
    dump.headers = dict(zip(header_tags, header_vals))
    dump.labels = labels

    return dump


def get_labels(ncol):
    if type(ncol) is c_int:
        ncol_py = ncol.value

    elif type(ncol) is int:
        ncol_py = ncol
        ncol = c_int(ncol)

    number_chars = lenlabel * ncol_py

    # Some Fortran compilers do not like passing an array of c_char[n]
    labels = (c_char * lenlabel * ncol_py)()

    libread.get_labels_c.argtypes = [POINTER(c_char * lenlabel * ncol_py), POINTER(c_int)]

    libread.get_labels_c(byref(labels), byref(ncol))

    labels = [str(labels[i].value.rstrip(), 'utf-8') for i in range(0, ncol.value)]

    return labels


def get_headers():

    headerval_length = c_int()
    headertag_length = c_int()

    libread.get_header_vals_size.argtypes = [POINTER(c_int), POINTER(c_int)]

    libread.get_header_vals_size(byref(headertag_length), byref(headerval_length))

    headertags = (c_char * ltags * headertag_length.value)()
    headervals = (c_double * headerval_length.value)()

    libread.get_headers.argtypes = [POINTER(c_char * ltags * headertag_length.value),
                                    POINTER(c_double * headerval_length.value),
                                    POINTER(c_int), POINTER(c_int)]

    libread.get_headers(byref(headertags), byref(headervals),
                        byref(headertag_length), byref(headerval_length))



    headertags = [str(headertags[i].value.rstrip(), 'utf-8')
                    for i in range(0, headertag_length.value)]


    headervals = np.ctypeslib.as_array(headervals)

    headertags_clean = list()
    headervals_clean = list()

    for i, headertag in enumerate(headertags):
        if headertag != '':
            headertags_clean.append(headertag)
            headervals_clean.append(headervals[i])

    return headertags_clean, headervals_clean



import pysplashsph
import os

print("********* FULL DUMP **************")

test_dir = os.path.dirname(os.path.realpath(__file__))

test_file_hdf5 = os.path.join(test_dir, 'data/fulldump_00000.h5')

data_hdf5 = pysplashsph.read.read_data(test_file_hdf5, filetype='Phantom', use_HDF5=True)

test_file_binary = os.path.join(test_dir, 'data/fulldump_00000')

data_binary = pysplashsph.read.read_data(test_file_binary, filetype='Phantom')

print("***** BINARY DATA HEADER ")

print(data_binary['header'])

data_binary_as_hdf5 = data_binary.as_hdf5

# print('Real HDF5', data_hdf5.keys())
# print('Real HDF5',data_hdf5['particles'].keys())
# print('My HDF5', data_binary_as_hdf5['particles'].keys())

print('Real HDF5 sinks', data_hdf5['sinks'].keys())
print('My HDF5 sinks', data_binary_as_hdf5['sinks'].keys())

# print('Real HDF5', data_hdf5['header'])
# print('Real HDF5', data_hdf5['header'].keys())
# print('Real HDF5', data_hdf5['header/Bextx'][()])
#
# print('My HDF5', data_binary_as_hdf5['header'])
# print('My HDF5', data_binary_as_hdf5['header'].keys())
# print('My HDF5', data_binary_as_hdf5['header/Bextx'][()])



print("********* SMALL DUMP **************")

test_dir = os.path.dirname(os.path.realpath(__file__))

test_file_hdf5 = os.path.join(test_dir, 'data/smalldump_00000.h5')

data_hdf5 = pysplashsph.read.read_data(test_file_hdf5, filetype='Phantom', use_HDF5=True)

test_file_binary = os.path.join(test_dir, 'data/smalldump_00000')

data_binary = pysplashsph.read.read_data(test_file_binary, filetype='Phantom')

data_binary_as_hdf5 = data_binary.as_hdf5

# print('Real HDF5', data_hdf5.keys())
# print('Real HDF5',data_hdf5['particles'].keys())
# print('My HDF5', data_binary_as_hdf5['particles'].keys())

print('Real HDF5 sinks', data_hdf5['sinks'].keys())
print('My HDF5 sinks', data_binary_as_hdf5['sinks'].keys())

for key in data_hdf5['particles'].keys():
    print(data_hdf5['particles'][key])

print('*******')

for key in data_binary_as_hdf5['header'].keys():
    print(data_binary_as_hdf5['header'][key])
    print(data_binary_as_hdf5['header'][key].value)


print(data_hdf5['header/massoftype'])

# print('Real HDF5', data_hdf5['header'])
# print('Real HDF5', data_hdf5['header'].keys())
# print('Real HDF5', data_hdf5['header/Bextx'][()])
#
# print('My HDF5', data_binary_as_hdf5['header'])
# print('My HDF5', data_binary_as_hdf5['header'].keys())
# print('My HDF5', data_binary_as_hdf5['header/Bextx'][()])



import os.path
from pathlib import Path
from numpy.ctypeslib import as_array

from ctypes import c_int, c_float, c_bool, byref
from . import _libexact as libexact
from ..utils import stdchannel_redirected


def checkfmt(str):
    return str.lower().replace(' ', '_').replace('-', '_').replace('/', '_')

def check_error(ierr, module):
    if type(ierr) is c_int:
        ierr = c_int.value
    if ierr == 1:
        print("Error in " + str(module) + ".")
        exit(1)

def shock(
    x,
    plot         = 'density',
    time         = 0.1,
    gamma        = 5./3.,
    xshock       = 0.,
    rho_L        = 1.,
    rho_R        = 0.125,
    p_L          = 1.,
    p_R          = 0.1,
    v_L          = 0,
    v_R          = 0.,
    rdust_to_gas = 0.):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    elif (pin == 'velocity'):
        iplot = 3
    elif (pin == 'uthermal'):
        iplot = 4
    elif (pin == 'deltav'):
        iplot = 5
    elif (pin == 'dustfrac'):
        iplot = 6
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || pressure || velocity || uthermal || deltav || dustfrac")
        exit(1)

    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._shock(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(xshock)),
            byref(c_float(rho_L)),
            byref(c_float(rho_R)),
            byref(c_float(p_L)),
            byref(c_float(p_R)),
            byref(c_float(v_L)),
            byref(c_float(v_R)),
            byref(c_float(rdust_to_gas)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "shock")

    return as_array(c_y)

def shock_sr(
    x,
    plot    = 'density',
    time    = 0.1,
    gamma   = 5./3.,
    rho_L   = 1.,
    rho_R   = 0.125,
    p_L     = 1.0,
    p_R     = 0.1,
    v_L     = 0.,
    v_R     = 0.):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    elif (pin == 'velocity'):
        iplot = 3
    elif (pin == 'uthermal'):
        iplot = 4
    elif (pin == 'density*'):
        iplot = 5
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || pressure || velocity || uthermal || density*")
        exit(1)

    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._shock_sr(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(rho_L)),
            byref(c_float(rho_R)),
            byref(c_float(p_L)),
            byref(c_float(p_R)),
            byref(c_float(v_L)),
            byref(c_float(v_R)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "shock_sr")

    return as_array(c_y)

def sedov(
    r,
    plot    = 'density',
    time    = 0.1,
    gamma   = 5./3.,
    rhozero = 1.,
    energy  = 1.,
    rmax    = 1.):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    elif (pin == 'uthermal'):
        iplot = 3
    elif (pin == 'kinetic_energy'):
        iplot = 4
    elif (pin == 'velocity'):
        iplot = 5
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || pressure || uthermal || kinetic_energy ||  velocity ")
        exit(1)

    ierr = 0
    c_r = (c_float*len(r))()
    c_y = (c_float*len(r))()
    c_r[:] = r[:]

    with stdchannel_redirected():
        libexact._sedov(
            byref(c_int(iplot)),
            byref(c_int(len(r))),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(rhozero)),
            byref(c_float(energy)),
            byref(c_float(rmax)),
            byref(c_r),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "sedov")

    return as_array(c_y)

def polytrope(
    r,
    gamma   = 5./3.,
    polyk   = 1.,
    totmass = 1.):


    ierr = 0

    c_r = (c_float*len(r))()
    c_y = (c_float*len(r))()

    c_r[:] = r[:]

    npartout = len(r)
    nout = c_int(0)

    with stdchannel_redirected():
        libexact._polytrope(
            byref(c_int(len(r))),
            byref(c_float(gamma)),
            byref(c_float(polyk)),
            byref(c_float(totmass)),
            byref(c_r),
            byref(c_y),
            byref(c_nout),
            byref(c_int(ierr))
        )

    check_error(ierr, "polytrope")

    return as_array(c_y)


def toystar1D(
    x,
    plot    = 'density',
    time    = 1.,
    gamma   = 5./3.,
    H0      = 1.,
    A0      = 1.,
    C0      = 1.,
    sigma   = 0.,
    norder  = -1):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    elif (pin == 'uthermal'):
        iplot = 3
    elif (pin == 'velocity_x'):
        iplot = 4
    elif (pin == 'mag_field_y'):
        iplot = 5
    elif (pin == 'ac_plane'):
        iplot = 7
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || pressure || uthermal || velocity_x ||  mag_field_y || ac_plane ")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._toystar1d(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(H0)),
            byref(c_float(A0)),
            byref(c_float(C0)),
            byref(c_float(sigma)),
            byref(c_int(norder)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "toystar1D")

    return as_array(c_y)


def toystar2D(
    x,
    plot    = 'density',
    time    = 1.,
    gamma   = 5./3.,
    polyk   = 1.,
    totmass = 1.,
    A0      = 1.,
    H0      = 1.,
    C0      = 1.,
    jorder  = -1,
    morder  = 0,
    V11     = 0,
    V22     = 0,
    V12     = 0,
    V21     = 0):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    elif (pin == 'uthermal'):
        iplot = 3
    elif (pin == 'velocity_x'):
        iplot = 4
    elif (pin == 'velocity_y'):
        iplot = 5
    elif (pin == 'x_vs_y'):
        iplot = 0
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || pressure || uthermal || velocity_x ||  velocity_y || x_vs_y ")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._toystar2d(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(polyk)),
            byref(c_float(totmass)),
            byref(c_float(A0)),
            byref(c_float(H0)),
            byref(c_float(C0)),
            byref(c_int(jorder)),
            byref(c_int(morder)),
            byref(c_float(V11)),
            byref(c_float(V22)),
            byref(c_float(V12)),
            byref(c_float(V21)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "toystar2D")

    return as_array(c_y)


def gresho(
    x,
    plot = 'velocity_phi'):

    pin = checkfmt(plot)
    if   (pin == 'velocity_phi'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = velocity_phi || pressure ")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._gresho(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "gresho")

    return as_array(c_y)

def mhdshock(
    x,
    plot     = 'density',
    solution = '7 jump',
    time     = 0.2,
    gamma    = 5./3.,
    xmin     = -1,
    xmax     = 1,
    xshock   = 0):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    elif (pin == 'velocity_x'):
        iplot = 3
    elif (pin == 'velocity_y'):
        iplot = 4
    elif (pin == 'velocity_z'):
        iplot = 5
    elif (pin == 'mag_field_y'):
        iplot = 6
    elif (pin == 'mag_field_z'):
        iplot = 7
    elif (pin == 'uthermal'):
        iplot = 8
    elif (pin == 'Bxzero'):
        iplot = 9
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || pressure || velocity_x ||  velocity_y || velocity_z || mag_field_y || mag_field_z || uthermal || Bxzero ")
        exit(1)

    ishk = 0
    pin = checkfmt(solution)
    if   (pin == 'brio_wu'):
        ishk = 1
    elif (pin == 'fast_slow'):
        ishk = 2
    elif (pin == '7_jump'):
        ishk = 3
    elif (pin == 'isothermal'):
        ishk = 4
    elif (pin == 'rarefaction'):
        ishk = 5
    elif (pin == 'mach_25'):
        ishk = 6
    elif (pin == 'toth'):
        ishk = 7
    else:
        print("PySplash Exact: Unrecognised solution type.")
        print("PySplash Exact: solution = Brio/Wu || fast/slow || 7_jump || isothermal ||  rarefaction || Mach_25 || Toth ")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]
    c_nout = c_int(0)

    with stdchannel_redirected():
        libexact._mhdshock(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_int(ishk)),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(xmin)),
            byref(c_float(xmax)),
            byref(c_float(xshock)),
            byref(c_x),
            byref(c_y),
            byref(c_nout),
            byref(c_int(ierr))
        )

    check_error(ierr, "mhdshock")

    return as_array(c_y)


def rhoh(
    x,
    plot        = 'density',
    ndim        = 3,
    hfact       = 1.2,
    pmassval    = 1.0):

    pin = checkfmt(plot)
    if   (pin == 'h'):
        iplot = 1
    elif (pin == 'density'):
        iplot = 2
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = h || density ")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._rhoh(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_int(ndim)),
            byref(c_float(hfact)),
            byref(c_float(pmassval)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "rhoh")

    return as_array(c_y)


def densityprofiles(
    x,
    plot    = 'density',
    profile = 'Plummer',
    Msphere = [1.0,0.0],
    rsoft   = [1.0,0.1]):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'potential'):
        iplot = 2
    elif (pin == 'force'):
        iplot = 3
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || potential || force ")
        exit(1)

    pin = checkfmt(profile)
    if   (pin == 'plummer'):
        iprofile = 1
    elif (pin == 'hernquist'):
        iprofile = 2
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: profile = Plummer || Hernquist ")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._densityprofiles(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_int(iprofile)),
            byref(c_float(Msphere[0])),
            byref(c_float(Msphere[1])),
            byref(c_float(rsoft[0])),
            byref(c_float(rsoft[1])),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "densityprofiles")

    return as_array(c_y)


def torus(
    x,
    plot        = 'density',
    torus       = 'default',
    Mstar       = 1.0,
    Rtorus      = 1.0,
    polyk       = 0.0764,
    distortion  = 1.1,
    gamma       = 5./3.):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'pressure'):
        iplot = 2
    elif (pin == 'uthermal'):
        iplot = 3
    elif (pin == 'Btheta'):
        iplot = 4
    elif (pin == 'Jphi_current'):
        iplot = 5
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density || pressure || uthermal || Btheta || Jphi_current")
        exit(1)

    itorus = 0
    pin = checkfmt(torus)
    if   (pin == 'default'):
        itorus = 1
    elif (pin == 'tokamak'):
        itorus = 2
    else:
        print("PySplash Exact: Unrecognised torus type.")
        print("PySplash Exact: torus = Default || Tokamak ")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._torus_(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_int(itorus)),
            byref(c_float(Mstar)),
            byref(c_float(Rtorus)),
            byref(c_float(polyk)),
            byref(c_float(distortion)),
            byref(c_float(gamma)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "torus")

    return as_array(c_y)


def ringspread(
    x,
    plot   = 'density',
    time   = 1.0,
    Mdisk  = 1.0,
    Rdisk  = 1.0,
    viscnu = 1.e-3):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    else:
        print("PySplash Exact: Unrecognised plot type.")
        print("PySplash Exact: plot = density")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._ringspread(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(Mdisk)),
            byref(c_float(Rdisk)),
            byref(c_float(viscnu)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "ringspread")

    return as_array(c_y)


def dustywave(
    x,
    plot       = 'gas_density',
    time       = 1.0,
    ampl       = 1.0,
    cs         = 1.0,
    Kdrag      = 1.0,
    lambdacoef = 1.0,
    x0         = 1.0,
    rhog0      = 1.0,
    rhod0      = 1.0):

    pin = checkfmt(plot)
    if   (pin == 'gas_velocity'):
        iplot = 1
    elif (pin == 'dust_velocity'):
        iplot = 2
    elif (pin == 'gas_density'):
        iplot = 3
    elif (pin == 'dust_density'):
        iplot = 4
    else:
        print("PySplash Exact: Unrecognised plot type `" + plot + "`")
        print("PySplash Exact: plot = gas_velocity || dust_velocity || gas_density || dust_density")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._dustywave(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(ampl)),
            byref(c_float(cs)),
            byref(c_float(Kdrag)),
            byref(c_float(lambdacoef)),
            byref(c_float(x0)),
            byref(c_float(rhog0)),
            byref(c_float(rhod0)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "dustywave")

    return as_array(c_y)


def rochelobe(
    x,
    primatypos = [0.,0.],
    secondarypos = [1.,0.],
    primarymass =  1.,
    secondarymass = 1.):


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._rochelobe(
            byref(c_int(len(x))),
            byref(c_float(primatypos[0])),
            byref(c_float(primatypos[1])),
            byref(c_float(secondarypos[0])),
            byref(c_float(secondarypos[0])),
            byref(c_float(primarymass)),
            byref(c_float(secondarymass)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "rochelobe")

    return as_array(c_y)


def cshock(
    x,
    plot='density',
    time=0.2,
    gamma=5./3.,
    machs = 50.,
    macha = 5.,
    xmin=-0.25,
    xmax= 0.25,):

    pin = checkfmt(plot)
    if   (pin == 'density'):
        iplot = 1
    elif (pin == 'mag_field_y'):
        iplot = 2
    elif (pin == 'velocity_x'):
        iplot = 3
    elif (pin == 'velocity_y'):
        iplot = 4
    elif (pin == 'mag_field_x'):
        iplot = 5
    else:
        print("PySplash Exact: Unrecognised plot type `" + plot + "`")
        print("PySplash Exact: plot = density || mag_field_x || mag_field_y || velocity_x || velocity_y")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._cshock(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(machs)),
            byref(c_float(macha)),
            byref(c_float(xmin)),
            byref(c_float(xmax)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "cshock")

    return as_array(c_y)


def check_spiral_params(i1, i2, j1, j2, nparams, nsolutions):
    if (i1 > i2):
        print("PySplash Exact: Wrong Spiral Params. i1 > i2. Should define blocks [i1,i2,j1,j2,val].")
        exit(1)
    if (j1 > j2):
        print("PySplash Exact: Wrong Spiral Params. j1 > j2. Should define blocks [i1,i2,j1,j2,val].")
        exit(1)
    if (i2 >= nparams):
        print("PySplash Exact: Wrong Spiral Params. i1 > "+str(nparams)+". Should define blocks [i1,i2,j1,j2,val].")
        exit(1)
    if (j2 >= nsolutions):
        print("PySplash Exact: Wrong Spiral Params. i1 > "+str(nsolutions)+". Should define blocks [i1,i2,j1,j2,val].")
        exit(1)

def planetdisc(
    x,
    plot='phi/r plane',
    spiral='Ogilvie/Rafikov',
    time=0.2,
    HonR = 0.05,
    rplanet = 1.,
    q_index = 0.25,
    narms = 1,
    spiral_params=[[1,1,0,6,360]]):
    # same as filling block i1 to i2, j1 to j2 with x
    # spiral_params = 0.
    # spiral_params(2,:) = 360.
    pin = checkfmt(plot)
    if   (pin == 'phi_r_plane'):
        iplot = 1
    elif (pin == 'x_y_plane'):
        iplot = 2
    else:
        print("PySplash Exact: Unrecognised plot type `" + plot + "`")
        print("PySplash Exact: plot = phi-r plane || x-y plane")
        exit(1)

    ispiral = 0
    pin = checkfmt(spiral)
    if   (pin == 'ogilvie_rafikov'):
        ispiral = 1
    elif (pin == 'spiral_arm_fitting'):
        ispiral = 2
    else:
        print("PySplash Exact: Unrecognised plot type `" + plot + "`")
        print("PySplash Exact: plot = Ogilvie/Rafikov || spiral_arm_fitting")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]
    nparams = 7
    nsolutions = 10
    c_params = (c_float*nparams*nsolutions)()
    for e in spiral_params:
        i1 = e[0]
        i2 = e[1]
        j1 = e[2]
        j2 = e[3]
        val = e[4]
        check_spiral_params(i1,i2,j1,j2,nparams,nsolutions)
        for i in range(i1,i2+1):
            for j in range(j1,j2+1):
                c_params[i][j] = val

    with stdchannel_redirected():
        libexact._planetdisc(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_int(ispiral)),
            byref(c_float(time)),
            byref(c_float(HonR)),
            byref(c_float(rplanet)),
            byref(c_float(q_index)),
            byref(c_int(narms)),
            byref(c_params),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "planetdisc")

    return as_array(c_y)


def bondi(
    x,
    plot = 'density',
    time = 10,
    gamma = 5./3.,
    const1 = 8.,
    const2 = 1.,
    Mstar = 1,
    relativistic  = True,
    geodesic_flow = False,
    is_wind       = True):

    pin = checkfmt(plot)
    if   (pin == 'velocity_x'):
        iplot = 1
    elif (pin == 'uthermal'):
        iplot = 2
    elif (pin == 'density'):
        iplot = 2
    else:
        print("PySplash Exact: Unrecognised plot type `" + plot + "`")
        print("PySplash Exact: plot = velocity_x || uthermal || density")
        exit(1)


    ierr = 0
    c_x = (c_float*len(x))()
    c_y = (c_float*len(x))()
    c_x[:] = x[:]

    with stdchannel_redirected():
        libexact._bondi(
            byref(c_int(iplot)),
            byref(c_int(len(x))),
            byref(c_float(time)),
            byref(c_float(gamma)),
            byref(c_float(const1)),
            byref(c_float(const2)),
            byref(c_float(Mstar)),
            byref(c_bool(relativistic)),
            byref(c_bool(geodesic_flow)),
            byref(c_bool(is_wind)),
            byref(c_x),
            byref(c_y),
            byref(c_int(ierr))
        )

    check_error(ierr, "bondi")

    return as_array(c_y)



"""
PySplashSPH
=====
**PySplashSPH** is a Python package that provides access to the Fortran
subroutines from the SPH visualisation code SPLASH.
Features
--------
- Plotting exact analytic functions for many hydrodynamical problems
- Reading many SPH code data formats
- Interpolation routines for SPH visualisation
Subpackages
-----------
- exact
    Contains functions for exact solutions to many hydrodynamical
    problems
- read
    Contains functions for reading data formats from many different
    SPH codes
- interpolation
    Contains functions for SPH interpolation
Documentation
-------------
See **** for documentation. The source code is
available at ****.
"""

# Canonical version number
__version__ = '0.0.2'

from . import exact, read, utils #, interpolation

__all__ = (['exact', 'read', 'utils'])



"""PySplashSPH setup.py."""

import io
import pathlib
import re
import inspect
import subprocess
import pathlib

from setuptools import setup
from setuptools.command.install import install
from setuptools.command.develop import develop
from wheel.bdist_wheel import bdist_wheel
from os import environ
from os.path import join, basename, isfile, abspath

src_dir = 'pysplashsph'

__version__ = re.search(
    r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]',  # It excludes inline comment too
    io.open(join(src_dir,'__init__.py'), encoding='utf_8_sig').read(),
).group(1)

install_requires = [
    'setuptools>=46.0.0',
    'numpy>=1.18.1',
    'h5py>=2.10.0',
    'pandas>=1.0.1',
]
packages = [
    'pysplashsph',
    'pysplashsph.exact',
    'pysplashsph.read',
    'pysplashsph.utils'
]

package_dir = {'pysplashsph': src_dir}
package_data = {"pysplashsph": ["libs/*.so*", "libs/*.dylib*"]}

description = 'Python wrapper module around SPLASH utilities.'
long_description = (pathlib.Path(__file__).parent / 'README.md').read_text()

splash_error = """
pysplashsph ERROR: Could not locate SPLASH directory
Please make sure that you have SPLASH installed in one of the following directories.

1. In the current directory, i.e. ./splash/
2. In the parent directory, i.e. ../splash/
3. In $SPLASH_DIR
4. In $HOME/splash

Note: you may have to do `git submodule update` if you're using the first option.
"""

def get_splash_dir():
    """
    Locate splash installation.

    Locations are checked in this order:
    1) Parent directory
    2) Environment variable $SPLASH_DIR
    3) $HOME/splash
    """
    current_dir = str(pathlib.Path(abspath(__file__)).parent)
    parent_dir  = str(pathlib.Path(current_dir).parent)
    home_splash = join(environ['HOME'], 'splash')
    cwd_splash  = join(current_dir, 'splash')

    if isfile(join(cwd_splash, 'Makefile')):
        splash_dir = cwd_splash
    elif 'splash' == basename(parent_dir) and isfile(join(parent_dir, 'Makefile')):
        splash_dir = parent_dir
    elif 'SPLASH_DIR' in environ and isfile(join(environ['SPLASH_DIR'], 'Makefile')):
        splash_dir = environ['SPLASH_DIR']
    elif isfile(join(home_splash, 'Makefile')):
        splash_dir = home_splash
    else:
        print(splash_error)
        exit(1)

    return splash_dir

splash_dir = get_splash_dir()

def build(splash_dir=splash_dir, compiler='gfortran', clean_first=False):
    libs = ['libexact', 'libread']

    print("\n>>> Building fortran source in directory: ", splash_dir, flush=True)

    errcode = 0

    if clean_first:
        errcode = subprocess.call(['make', 'clean'], cwd=splash_dir)
        if errcode != 0:
            print('pysplashsph ERROR:')
            print('Could not "make clean"')
            exit(1)

    for lib in libs:
        print("\nBuilding {}:".format(lib), flush=True)
        errcode = subprocess.call(['make','SYSTEM={}'.format(compiler),lib], cwd=splash_dir)
        if errcode != 0:
            print('pysplashsph ERROR:')
            print('Could not build library.')
            exit(1)

        print('\nCopying {}.so to pysplashsph/libs/. \n'.format(lib), flush=True)
        errcode = subprocess.call(['cp', join(splash_dir,'build/{}.so'.format(lib)), join(src_dir, 'libs/.')])
        if errcode != 0:
            print('pysplashsph ERROR:')
            print('Could not copy library.')
            exit(1)

    print("<<< Finished building fortran source.\n", flush=True)


"""
Build the fortran library first, then do regular run()
"""
class custom_bdist_wheel(bdist_wheel):
    def run(self):
        build()
        bdist_wheel.run(self)

class custom_install(install):
    def run(self):
        # Don't recompile if bdist_wheels was already run
        if self._called_from_setup(inspect.currentframe()):
            build()
        install.run(self)

class custom_develop(develop):
    def run(self):
        # Don't recompile if uninstalling
        if not self.uninstall:
            build()
        develop.run(self)

print('\n>>>>> running setup.py >>>>>', flush=True)
setup(
    name='pysplashsph',
    version=__version__,
    author='Josh Calcino',
    author_email='josh.calcino@gmail.com',
    url='https://github.com/joshcalcino/pysplashsph',
    description=description,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=packages,
    package_dir=package_dir,
    package_data=package_data,
    include_package_data=True,
    license='MIT',
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS",
        "Topic :: Scientific/Engineering :: Astronomy",
        "Topic :: Scientific/Engineering :: Visualization",
    ],
    cmdclass={
        'install': custom_install,
        'bdist_wheel': custom_bdist_wheel,
        'develop': custom_develop,
    },
)
print('<<<<< end running setup.py <<<<<\n', flush=True)



import os
import sys
from contextlib import contextmanager
import tempfile
import warnings


@contextmanager
def stdchannel_redirected(print_warnings=False, stdchannel=sys.stdout):
    """
    Redirect stdout from Fortran code into a temporary file, so to not clutter
    the terminal with unnecessary print statements.
    """

    # Create a temporary file to store stdout from Fortran code
    dest_file = tempfile.NamedTemporaryFile(mode='w')

    try:
        oldstdchannel = os.dup(stdchannel.fileno())

        # Redirect stdchannel messages into our temporary file
        os.dup2(dest_file.fileno(), stdchannel.fileno())

        # Allow Fortran code to run
        yield

    finally:
        if oldstdchannel is not None:
            os.dup2(oldstdchannel, stdchannel.fileno())
        if dest_file is not None:
            with open(dest_file.name, 'r') as f:
                # Check the output for errors
                stdchannel_check_errors(f, print_warnings=print_warnings)
            dest_file.close()


def stdchannel_check_errors(fileobject, print_warnings=False):
    """ Simple function to check a fileobject in read mode for any warnings or
    errors.
    """

    lines = fileobject.readlines()
    warning_messages = []
    error_messages = []
    for line in lines:
        if 'warning' in line.lower():
            warning_messages.append(line)
        elif 'error' in line.lower():
            error_messages.append(line)

    if len(warning_messages) > 0 and print_warnings:
        warnings.warn("The following warnings were encountered in SPLASH", RuntimeWarning)
        for warning in warning_messages:
            print(warning.strip())

    if len(error_messages) > 0:
        for error in error_messages:
            print(error.strip())
        raise RuntimeError("Errors encountered in SPLASH")
